#========================================================================================
# package      DE_EPAGES::Customer::Hooks::User
# state        public
# description  hooks triggered by user functions
#========================================================================================
package DE_EPAGES::Customer::Hooks::User;

use strict;
use DE_EPAGES::Customer::API::Shop      qw ( GetCustomerForUser );

#========================================================================================
# function     OnConfirmUserAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnConfirmUserAccount( $hParams );
#----------------------------------------------------------------------------------------
# description  Set customer billing address email to users billing address email
#               (Hook UI_ConfirmUserAccount)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet</li>
#                   <li>User - user object</li>
#                   <li>EMail - email address</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnConfirmUserAccount {
    my ($hParams) = @_;
    my $User = $hParams->{'User'};
    my $Shop = $User->getSite();
    my $EMail = $hParams->{'EMail'};

    my $Customer = GetCustomerForUser($User, $Shop);
    return unless $Customer;

    my $CustomerBillingAddress = $Customer->get('BillingAddress');
    return unless $CustomerBillingAddress;

    $CustomerBillingAddress->set({'EMail' => $EMail});

    return;
}

1;

