#========================================================================================
# package      DE_EPAGES::Ebay::Hooks::EbayAuction
# state        public
# description  hook functions for EbayAuction
#========================================================================================
package DE_EPAGES::Ebay::Hooks::EbayAuction;
use base qw( Exporter );

use strict;

#========================================================================================
# function     OnAfterUpdateEbayAuction
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateEbayAuction( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateEbayAuction (OBJ_AfterUpdateEbayAuction)
#               is triggered after an EbayAuction is updated.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object            - EbayAuction object            - object
#                 <li>Values            - updated values                - ref.hash
#                 <li>AuctionID         - auction id                    - string
#                 <li>LanguageID        - language id                   - string
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateEbayAuction {
    my ($hParams) = @_;

    # set some attributes on the sub auctions if changed on the parent auction
    my $EbayAuction = $hParams->{'Object'};
    return unless defined $EbayAuction;
    
    my $hNewValues                      = $hParams->{'Values'};
    my $EbayCatalogMappingRequired      = $hNewValues->{'EbayCatalogMappingRequired'};
    my $EbayCatalogBrandMappingRequired = $hNewValues->{'EbayCatalogBrandMappingRequired'};
    return unless (defined $EbayCatalogMappingRequired or defined $EbayCatalogBrandMappingRequired);
   
    my $hSubAuctions = $EbayAuction->get('SubAuctions');
    return unless scalar(@$hSubAuctions);

    foreach my $SubAuction (@$hSubAuctions){
        foreach my $Attribute (qw(EbayCatalogMappingRequired EbayCatalogBrandMappingRequired)){
            $SubAuction->set({$Attribute => $hNewValues->{$Attribute}}) if exists $hNewValues->{$Attribute} ;
        }
    }
}

1;