use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw (LoadObject);
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function     ChangeTemplatesUrlHttpToHttps
# state        public
#----------------------------------------------------------------------------------------
# syntax       ChangeTemplatesUrlHttpToHttps($$hOptions)
#----------------------------------------------------------------------------------------
# description  Replaces server path url to https.
#----------------------------------------------------------------------------------------
# input        $Verbose      | verbose mode   | boolean
#========================================================================================

sub ChangeTemplatesUrlHttpToHttps($) {
    my ($Verbose) = @_;

    my $dbi = GetCurrentDBHandle();
    my $Object;
    my $ChangedCounter = 0;
    my $aaResult = $dbi->execute('select ebayusertemplateid, content from ebayusertemplate');

    foreach my $aResult (@$aaResult) {
        $Object = LoadObject($aResult->[0]);

        # Load value of 'content' for current ebayusertemplate,
        local $_ = $Object ->get('Content');
        my $Changed = 0;

        # Replace string in temporary variable $_ if found.
        $Changed = 1 if s/#Shop.ProtocolAndServer#WebPath/#Shop.ProtocolAndServerSSL#WebPath/g;
        if ($Changed) {
            $Object->set({'Content'=>$_});
            $ChangedCounter++;
        }
    }
    printf 'Changed Templates: %4d', $ChangedCounter if $Verbose;
    print "\n" if $Verbose;
}

sub Main {

    my $Password;
    my $StoreName;
    my $Help;
    my $Verbose;

    GetOptions(
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'help'          => \$Help,
        'verbose'       => \$Verbose,
    );
    usage() if $Help;
    unless($StoreName) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }

    my $time1 = time;
    RunOnStore(
        Store       => $StoreName,
        DBPassword  => $Password,
        Sub         => sub  {
            ChangeTemplatesUrlHttpToHttps($Verbose);
            SynchronizeCache();
        },
    );
    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password                        ()
    -storename  name of store                                 required

flags:
    -help       show the command line options
    -verbose    print debug infos

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
