#========================================================================================
# package      DE_EPAGES::Ebay::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to insert products for ebayauction from new object tray.
#========================================================================================
package DE_EPAGES::Ebay::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Ebay::API::EbayAuction qw ( InsertNewAuctionFromProduct );
use DE_EPAGES::Ebay::API::Object::EbayCategoryTree;
use DE_EPAGES::Ebay::API::Object::RemoteEbay;
use DE_EPAGES::Ebay::API::Object::RemoteRESTEbay;
use DE_EPAGES::Ebay::API::Tools qw(GetAuctionsSellerProfiles);
use JSON qw( to_json );

sub JSONInsertAuctionProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Category = $Servlet->object;
    my $Shop = $Category->getSite;
    my $Form = $Servlet->form;

    my @ProductIDs = $Form->value('ProductIDs');
    my $aProducts = $Shop->loadObjects(\@ProductIDs, 'Product');

    my $hhBusinessPolicySellerProfiles = GetAuctionsSellerProfiles([$Shop->get('EbayDefaultAuction')]);

    foreach my $Product (@$aProducts) {
        eval {
            InsertNewAuctionFromProduct(
                $Shop,
                $Product,
                undef,
                $Shop->get('EbayDefaultAuction')->getSellerProfile($hhBusinessPolicySellerProfiles)
            );
        };
        if (ExistsError()) {
            my $Error = GetError();

            Error($Error) unless $Error->code ~~ [qw(
                CanNotAuctionDownloadProduct CanNotAuctionMasterProduct NoShopEnvironment
            )];
            $Form->mapFormError({'Name' => 'AuctionProductsFromTray'}, $Error);
        }
    }
    $Form->executeFormError;
    return;
}

sub GetCategoriesJson {
    my $self = shift;
    my ($Servlet) = @_;

    my $Auction = $Servlet->object;
    my $ShopEnv = $Auction->get('EbayShopEnvironment');

    my $hResponse;
    my @Errors;
    my @CategoriesSorted;
    my $ParentCategory;

    push @Errors, {
        'reason'    => 'EbayWarningNoEbayEnvironment'
    } unless defined $ShopEnv;

    if (defined $ShopEnv) {
        # get parent category from query
        my $Form             = $Servlet->form;
        my $ParentCategoryID = $Form->value('ParentCategoryID');

        # set parent to root
        $ParentCategoryID = 0 unless $ParentCategoryID;

        #load categories
        my @Categories;
        my $EbayCategoryTree = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($ShopEnv->get('EbayEnvironment'));

        if (not $ParentCategoryID) {
            #get root categories
            push @Categories, @{ $EbayCategoryTree->getInfoRootCategories() };
        } else {
            #get sub categories of parent
            my $Subs = $EbayCategoryTree->getInfoEbayCategoriesByParentCategoryID( $ParentCategoryID );
            push @Categories, @$Subs;
        }

        @CategoriesSorted = sort{ "\U".$a->{Name} cmp "\U".$b->{'Name'} }(@Categories);

        # load current category
        $ParentCategory           = $EbayCategoryTree->getInfoEbayCategory( $ParentCategoryID ) if $ParentCategoryID;
        $ParentCategory->{'Path'} = $EbayCategoryTree->getCategoryPath($ParentCategoryID );
    }

    if (scalar @Errors) {
        $hResponse = {
            'Success'    => JSON::false,
            'doneErrors' => \@Errors
        };
    } else {
        $hResponse = {
            'Success'         => JSON::true,
            'CategoryList'    => \@CategoriesSorted,
            'CurrentCategory' => $ParentCategory
        };
    }

    return $self->writeContentOutput($hResponse);
}

sub GetInternalCategoryIdJson {
    my $self = shift;
    my ($Servlet) = @_;

    my $hResponse;
    my $Auction            = $Servlet->object;
    my $ExternalCategoryId = $Servlet->form->value('ExternalCategoryID');
    my $ShopEnv            = $Auction->get('EbayShopEnvironment');
    my $EbayCategoryTree   = DE_EPAGES::Ebay::API::Object::EbayCategoryTree->new($ShopEnv->get('EbayEnvironment'));
    my $Category           = $EbayCategoryTree->getInfoEbayCategoryByExternalCategoryId($ExternalCategoryId);

    if (defined $Category and $Category->{'IsLeaf'}) {
        my $ParentCategoryID   = $Category->{'ParentCategoryID'};
        my $InternalCategoryId = $Category->{'EbayCategoryID'};
        $hResponse = {
            'Success' => JSON::true,
            'ParentCategoryID' => $ParentCategoryID,
            'EbayCategoryID'   => $InternalCategoryId
        };
    } else {
        $hResponse = {
            'Success'    => JSON::false,
            'doneErrors' => 'NoLeafId'
        };

    }
    return $self->writeContentOutput($hResponse);
}

sub SearchCatalogProductByGtin {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $Gtin = $Form->value('SearchString');
    return $self->writeContentOutput( {'success' => JSON::false}) unless $Gtin;
    my $RemoteRestEbay   = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new(
                          'ShopEnvironment' => $Servlet->object->get('EbayShopEnvironment')
                          );

    my $hResponse;
    eval {
        $hResponse = $RemoteRestEbay->searchProductByGtin($Gtin);
    };

    if(ExistsError()){
        my $Error = GetError();
        Error($Error) unless ( $Error->code eq 'API_CALL_ERROR' );
        $self->writeContentOutput({'success' => JSON::false,
                                   'error'   => 'ApiCallError'        
                                 });
    }

    my $hJSONResponse = {
        'success'    => JSON::true,
        'products'   => defined $hResponse->{'productSummaries'} ? to_json($hResponse->{'productSummaries'}) : to_json([])
    };
   
    return $self->writeContentOutput($hJSONResponse);

}

1;
