package DE_EPAGES::EbayProvider::UI::Store;
use base 'DE_EPAGES::Presentation::UI::Object';

use strict;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw(GetLog);
use DE_EPAGES::Core::API::PerlTools qw ( VerifySecureCode CreateSecureCode );
use DE_EPAGES::Core::API::Url qw(UrlEncodeValue);
use DE_EPAGES::Database::API::Connection                    qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Table::Currency                 qw(GetInfoCurrency);
use DE_EPAGES::Object::API::Table::Language                 qw(GetInfoLanguagesByCode2);
use DE_EPAGES::Object::API::Factory                         qw(LoadRootObject);
use DE_EPAGES::XML::API::SOAPDateTimeFormatXML;
use DE_EPAGES::EbayProvider::API::Object::RemoteStore;
use DE_EPAGES::EbayProvider::API::Constants;#EBAY_OAUTH_SCOPE

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $Store = $Servlet->object;

    my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
        'Store' => $Store,
    );
    my $hValues = $RemoteStore->get( [qw (EbayRuName EbayDevId EbayAppId EbayCertId
                                          EbaySandBoxRuName EbaySandBoxDevId EbaySandBoxAppId EbaySandBoxCertId
                                          EbayApiVersion EbayEnableLogging
                                          EbayProviderToken EbayProviderTokenExpiryDate
                                          EbayProviderOAuthToken EbayProviderOAuthTokenExpiryDate
                                          EbaySandBoxProviderToken EbaySandBoxProviderTokenExpiryDate
                                          EbaySandBoxProviderOAuthToken EbaySandBoxProviderOAuthTokenExpiryDate) ] );

    my $parser = DE_EPAGES::XML::API::SOAPDateTimeFormatXML->new();
    my $SystemTimeZone = LoadRootObject()->get('TimeZone');

    for (qw(EbayProviderTokenExpiryDate EbayProviderOAuthTokenExpiryDate EbaySandBoxProviderTokenExpiryDate EbaySandBoxProviderOAuthTokenExpiryDate)) {
        eval {
            $hValues->{$_} = $parser->parse_datetime($hValues->{$_},  $SystemTimeZone) if $hValues->{$_};
        };
        if (ExistsError()) {
            GetLog()->debug('Wrong DateTime Format: '.GetError()->message);
        }
    }

    foreach my $value (keys %$hValues) {
        $Servlet->vars( $value, $hValues->{$value}  );
    }

    my $aEnvironments = $RemoteStore->getEbayEnvironment();
    $Servlet->vars( 'EbayEnvironments', $aEnvironments );
    $Servlet->vars( 'EbaySandBoxSettingsComplete',
        ($hValues->{'EbaySandBoxRuName'} and
         $hValues->{'EbaySandBoxDevId'} and
         $hValues->{'EbaySandBoxAppId'} and
         $hValues->{'EbaySandBoxCertId'} and
         $hValues->{'EbaySandBoxProviderToken'}
        )? 1:0);

    $Servlet->vars( 'EbayProductionSettingsComplete',
        ($hValues->{'EbayRuName'} and
         $hValues->{'EbayDevId'} and
         $hValues->{'EbayAppId'} and
         $hValues->{'EbayCertId'} and
         $hValues->{'EbayProviderToken'}
        )? 1:0);

    my ($US_Environment, $SANDBOX_Environment);
    for my $env (@$aEnvironments) {
        for (qw(CategoryListUpdateTimeEbay CategoryListUpdateTime)) {
            eval {
                $env->{$_} = $parser->parse_datetime($env->{$_},  $SystemTimeZone) if $env->{$_};
            };
            if (ExistsError()) {
                GetLog()->debug('Wrong DateTime Format: '.GetError()->message);
            }
        }
        # load currencies and languages
        $env->{'Language'} = GetInfoLanguagesByCode2(lc $env->{'LanguageID'})->[0];
        $env->{'Currency'} = GetInfoCurrency($env->{'CurrencyID'});

        if ($env->{'Alias'} eq 'US') {
            $US_Environment = $env;
        } elsif ($env->{'Alias'} eq 'SANDBOX') {
            $SANDBOX_Environment = $env;
        }

    }

    # here we are on the site db in /Stores/Store
    my $StoreURL = $Store->get('WebUrlAdminSSL');
    my $SiteRedirectURL = $StoreURL   ."&ViewAction=PBO-SaveEbayToken";
    my $SecurityCode = CreateSecureCode($SiteRedirectURL.$Store->get('GUID'));

    # add the correct SignInPrams for each environment
    # we identify the correct env by adding "SB" or "US"
    my $US_SignInParams;
    if ($hValues->{'EbayAppId'} and $hValues->{'EbayRuName'} ){
        $US_SignInParams = '?client_id='   . $hValues->{'EbayAppId'}  .
                            '&redirect_uri='  . $hValues->{'EbayRuName'} .
                            '&response_type=' . 'code' .
                            '&scope='         . EBAY_OAUTH_SCOPE.
                            '&state='         . UrlEncodeValue($SiteRedirectURL.'&sec='.$SecurityCode.'&env=US','-'); 
    }

    my $SANDBOX_SignInParams;
    if($hValues->{'EbaySandBoxAppId'} and $hValues->{'EbaySandBoxRuName'} ){
        $SANDBOX_SignInParams = '?client_id='     . $hValues->{'EbaySandBoxAppId'} .
                                '&redirect_uri='  . $hValues->{'EbaySandBoxRuName'} .
                                '&response_type=' . 'code' .
                                '&scope='         . EBAY_OAUTH_SCOPE.
                                '&state='         . UrlEncodeValue($SiteRedirectURL.'&sec='.$SecurityCode.'&env=SANDBOX','-'); 
    }

    $Servlet->vars('EbaySignInParams', $US_SignInParams);
    $Servlet->vars('EbaySandBoxSignInParams', $SANDBOX_SignInParams);
    $Servlet->vars('EbaySignInUrl', $US_Environment->{'OAuthGrandApplicationAccessEndpoint'});
    $Servlet->vars('EbaySandBoxSignInUrl', $SANDBOX_Environment->{'OAuthGrandApplicationAccessEndpoint'});

    return;
}

# Sample:
#https://<server>/epages/Site.admin/?ObjectPath=/Stores/Store
#&ViewAction=PBO-SaveEbayToken
#&sec=55ce0e878ea9f31ed0616c2ee87d5d9c
#&code=v%5E1.1%23i%5E1%23I%5E3%23p%5E3%23f%5E0%23r%5E1%23t%5EUl41Xzk6QjhCQTg5OTJFQUMzMDJBNjExNjI4Mjc2QjNDN0E4QzlfMl8xI0VeMTI4NA#%3D%3D
#&expires_in=299&env=SANDBOX' 
sub SaveEbayToken {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;

    my $Store    = $Servlet->object;
    my $StoreURL = $Store->get('WebUrlAdminSSL');

    my $hFormValues = {};
    foreach my $Values (qw(sec code expires_in env)) {
        $hFormValues->{$Values} = $Form->queryValue($Values);
        $self->_showProviderEbayPage($Servlet, $Store, 1)  unless $hFormValues->{$Values};
    }

    #verify security code
    my $StoreRedirectURL = $StoreURL   ."&ViewAction=PBO-SaveEbayToken";

    $self->_showProviderEbayPage($Servlet, $Store, 1) unless VerifySecureCode($StoreRedirectURL.$Store->get('GUID'), $hFormValues->{'sec'});


    # get the token by calling a RemoteEbay function
    my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new('Store' => $Store);

    # get the tokens by using the webservice method
    my $result = {};
     eval {
        $result = $RemoteStore->getOAuthTokensByAuthCode( $hFormValues->{'code'} ,$hFormValues->{'env'} );
     };

    if (ExistsError()){
        my $Error = GetError();
        my $Message = $Error->shortMessage;
        GetLog()->debug('fetchToken failed: '.$Message);
        $self->_showProviderEbayPage($Servlet, $Store, 1)
    } else {
        # save token for the live or sandbox provider attributes on the store
        $RemoteStore->saveOAuthToken( $result ,$hFormValues->{'env'} );
    }


    $Servlet->vars('SUCCESS_SAVE_TOKEN', 1);

    $self->_showProviderEbayPage($Servlet, $Store,0);
}

sub _showProviderEbayPage {
    my $self = shift;
    my ($Servlet, $Store, $IsError) = @_;

    my $PageArea = $Servlet->form->queryValue('env') eq 'SANDBOX' ? 'SandBoxArea' : 'LiveArea';
    $Servlet->vars('DialogArea', $PageArea);

    $Servlet->vars('ERR_FETCH_TOKEN', 1) if $IsError;

    # show the ebay environment page
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError'    => 0,
            'ViewObjectID' => $Store->id,
        },
        'ViewAction' => 'PBO-ViewEbayToken'
    });
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $Store = $Servlet->object;

    my $hFormValues = $Servlet->form->formValues;

    my $hValues = {};

    for (qw(EbayRuName EbayDevId EbayAppId EbayCertId
            EbaySandBoxRuName EbaySandBoxDevId EbaySandBoxAppId EbaySandBoxCertId
            EbayApiVersion EbayEnableLogging))
    {
        $hValues->{$_} = $hFormValues->{$_}->[0] if defined $hFormValues->{$_}->[0];
    }

    # remove whitespace
    for (qw(EbayRuName EbayDevId EbayAppId EbayCertId EbaySandBoxRuName EbaySandBoxDevId EbaySandBoxAppId EbaySandBoxCertId )) {
        $hValues->{$_} =~ s/^\s*(.*?)\s*$/$1/g;
    }

    my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
        'Store' => $Store,
    );
    $RemoteStore->set( $hValues ) if keys %$hValues;
}

sub CheckEbayToken {
    my $self = shift;
    my ($Servlet) = @_;

    my $Store = $Servlet->object;
    my $hFormValues = $Servlet->form->formValues;

    $Servlet->vars('DialogArea',$hFormValues->{'DialogArea'}->[0]);

    eval {
        my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
            'Store' => $Store,
        );
        my $result = $RemoteStore->checkEbayToken( ($hFormValues->{'DoTestOnSandBox'}->[0])?'SANDBOX':'US' );

        if (not $result) {
            Error('TestNotSuccessful');
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        my $Message = $Error->message;

        GetLog()->debug('EbayToken not valid: '.$Message);

        $Servlet->vars('ERR_TOKEN',$Message);

        my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
            'Store' => $Store,
        );
       # remove Token because if the test fails we have problems also with the refresh token
        if ($hFormValues->{'DoTestOnSandBox'}->[0]) {
            $RemoteStore->set({'EbaySandBoxProviderToken'                => undef,
                               'EbaySandBoxProviderTokenExpiryDate'      => undef,
                               'EbaySandBoxProviderOAuthToken'           => undef,
                               'EbaySandBoxProviderOAuthTokenExpiryDate' => undef,
                              });
        } else {
            $RemoteStore->set({'EbayProviderToken'                => undef,
                               'EbayProviderTokenExpiryDate'      => undef,
                               'EbayProviderOAuthToken'           => undef,
                               'EbayProviderOAuthTokenExpiryDate' => undef,
                              });
        }
    }
    $Servlet->vars('TOKENTESTED',1);

    return;
}

sub DeleteEbayToken {
    my $self = shift;
    my ($Servlet) = @_;

    my $Store = $Servlet->object;
    my $hFormValues = $Servlet->form->formValues;

    $Servlet->vars('DialogArea',$hFormValues->{'DialogArea'}->[0]);

    eval {

        my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
            'Store' => $Store,
        );
        my $now = GetCurrentDBHandle()->currentDateTime;

        if (not $hFormValues->{'DoTestOnSandBox'}->[0]) {
            if (not $hFormValues->{'ACCEPTLIVE'}->[0]) {
                $Servlet->vars('DELETELIVE',1);
                $Servlet->vars('DoTestOnSandBox',0);
                return;
            } else {
                $RemoteStore->set({'EbayProviderToken' => undef});
                $RemoteStore->set({'EbayProviderTokenExpiryDate' => $now});
                $RemoteStore->set({'EbayProviderOAuthToken' => undef});
                $RemoteStore->set({'EbayProviderOAuthTokenExpiryDate' => $now});
                $Servlet->vars('LIVETOKEN_DELETED',0);
            }
        }else{
            if (not $hFormValues->{'ACCEPTSB'}->[0]) {
                $Servlet->vars('DELETESB',1);
                $Servlet->vars('DoTestOnSandBox',1);
                return;
            } else {
                $RemoteStore->set({'EbaySandBoxProviderToken' => undef});
                $RemoteStore->set({'EbaySandBoxProviderTokenExpiryDate' => $now});
                $RemoteStore->set({'EbaySandBoxProviderOAuthToken' => undef});
                $RemoteStore->set({'EbaySandBoxProviderOAuthTokenExpiryDate' => $now});
                $Servlet->vars('SBTOKEN_DELETED',0);
            }
        }
    };

    if (ExistsError()) {
        my $Error = GetError();
        my $Message = $Error->message;

        GetLog()->debug('Error DeleteEbayToken: '.$Message);
    }

    return;
}

sub GetCategories {
    my $self = shift;
    my ($Servlet) = @_;

    my $Store = $Servlet->object;
    my $Form = $Servlet->form;
    my $hValues = $Form->form($Store, 'ListedObjects');

    my @ListObjectIDs = map { $_->{'ListObjectID'} }
                        @{$hValues->{'ListObjectIDs'}};

    return unless @ListObjectIDs;

    my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
        'Store' => $Store,
    );
    $RemoteStore->getCategories(\@ListObjectIDs);

    sleep(10);
    return 1;
}

sub SetEnvironmentVisible {
    my $self = shift;
    my ($Servlet, $SetVisible) = @_;

    $SetVisible = 1 unless defined $SetVisible;

    my $Store = $Servlet->object;
    my $Form = $Servlet->form;
    my $hValues = $Form->form($Store, 'ListedObjects');

    my @ListObjectIDs = map { $_->{'ListObjectID'} }
                        @{$hValues->{'ListObjectIDs'}};

    my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
        'Store' => $Store,
    );
    foreach my $EnvID (@ListObjectIDs) {
        $RemoteStore->objectService->set( $EnvID, {'IsVisible' => $SetVisible} )->result;
    }

    return;
}

sub SetEnvironmentInvisible {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->SetEnvironmentVisible($Servlet, 0);
}

sub ViewEbayStatistics {
    my $self = shift;
    my ($Servlet) = @_;

    my $Store = $Servlet->object;

    my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
        'Store' => $Store,
    );

    my $aCallType = $RemoteStore->getCallTypes();
    $Servlet->vars('EbayCallTypes' => $aCallType);

    my $aEnvironments = $RemoteStore->getEbayEnvironment();
    $Servlet->vars( 'EbayEnvironments', $aEnvironments );
}


sub GetEbayStatistics {
    my $self = shift;
    my ($Servlet) = @_;

    my $Store = $Servlet->object;

    my $parser = DE_EPAGES::XML::API::SOAPDateTimeFormatXML->new();
    my $SystemTimeZone = LoadRootObject()->get('TimeZone');
    my $hFormValues = $Servlet->form->form($Store, 'CallStatistics');

    $Servlet->vars('DateFrom' => $hFormValues->{'DateFrom'});
    $Servlet->vars('DateTo' => $hFormValues->{'DateTo'});
    $Servlet->vars('Interval' => $hFormValues->{'Interval'});
    $Servlet->vars('IntervalType' => $hFormValues->{'IntervalType'});
    $Servlet->vars('EbayEnvironment' => $hFormValues->{'EbayEnvironment'});
    $Servlet->vars('EbayCallType' => $hFormValues->{'EbayCallType'});

    # set DateTo to late evening
    $hFormValues->{'DateTo'} = $hFormValues->{'DateTo'} + DateTime::Duration->new(hours => 23, minutes => 59, seconds => 59);

    $hFormValues->{$_} = $parser->format_datetime($hFormValues->{$_}, $SystemTimeZone) for (qw(DateFrom DateTo));

    my $RemoteStore = DE_EPAGES::EbayProvider::API::Object::RemoteStore->new(
        'Store' => $Store,
    );
    my $result = $RemoteStore->getCallStatistics($hFormValues);
    my $maxValue = 1;

    foreach (@$result) {
        $_->{'StartDate'} = $parser->parse_datetime($_->{'StartDate'}, $SystemTimeZone);
        $maxValue = $_->{'Count'} if ($maxValue < $_->{'Count'});
    }

    $Servlet->vars('Statistics' => $result);
    $Servlet->vars('MaxValue' => $maxValue);
}


1;
