package DE_EPAGES::EbayProvider::UI::System;
use base 'DE_EPAGES::Presentation::UI::Object';

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use URI::Escape qw( uri_escape_utf8 uri_unescape );
use MIME::Base64;

# central entry point for the OAuth2 authentication workflow
# this function is called by ebay after the customer has grant the permissions for the app
# the auth code for fetching the token is given by ebay here
# here we extract the target url from state parameter, adds the code  and the expiration and 
# forward to the url given by state
# https://mhoehn-vm1-linux.intern.epages.de/epages/Site.admin?
# ViewAction=OAuth&
# state=https%3A%2F%2Fdemounityde.mhoehn-vm1-linux.intern.epages.de%2Fepages%2FDemoUnityDE.admin
# %2F%3FObjectPath%3D%2FShops%2FDemoUnityDE%2FEbayShopEnvironments%2FEbayShopEnvironment1%26ViewAction%3DMBO-SaveEbayToken%26sec%3D4b711b790c3ca015f18ee9720c5a6bf9
# &error=access_denied
sub NewEbayOAuthAuthentication {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;
    my $hFormValues = {};
    for (qw(state code expires_in error)) {
        $hFormValues->{$_} = $Form->queryValue($_);
    }

    # inside the state all values for routing the request are stored
    my $State = uri_unescape($hFormValues->{'state'});
    
    if($hFormValues->{'error'}){
        # add only the error...
        $State .= '&error=' . $hFormValues->{'error'};
    } else {

# Todo add error informations here

        Error ('InvalidRequest') unless defined $State;
        Error ('InvalidRequest') unless defined $hFormValues->{'code'};
        # add the expiration info and the code for fetching the token
        $State .= '&code=' .uri_escape_utf8($hFormValues->{'code'}). '&expires_in=' . $hFormValues->{'expires_in'};
    }

    # simply forward to state (can be the Site or the Store (Shop))
    $Servlet->vars('ExitAfterEvent', 1);
    my $Response = $Servlet->response;
    $Response->code(303);
    $Response->header('Location', $State);
}

1;

