#==============================================================================
# package      DE_EPAGES::MailType::UI::Site
# state        private
#==============================================================================
package DE_EPAGES::MailType::UI::Site;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::MailType::API::NewPassword   qw( SendNewPasswordToken );
use DE_EPAGES::Object::API::Factory         qw( LoadRootObject );
use DE_EPAGES::Presentation::API::User      qw( GetUsersByEMail );

sub SendUserNewPassword {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->testSecureConnection();
    $Servlet->testPostRequest();

    my $Site = $Servlet->object;
    my $hFormValues = $Servlet->form->form($Site, 'NewPassword');
    my $Email = $hFormValues->{'Login'};

    # Spam protection
    # Checks if honeypot form field 'SenderName' is set
    if($Servlet->form->value('SenderName')) {
        $Servlet->vars('ViewAction', 'View');
        return;
    }

    my $aUsers = GetUsersByEMail($Email, $Site);

    # Check if user with this login exists.
    unless (scalar @{$aUsers}) {
        $Servlet->form->executeFormError({
            'Reason'=> 'LOGIN_NOT_FOUND',
            'Form'  => 'NewPassword',
            'Name'  => 'Login',
            'Value' => $hFormValues->{'Login'},
        });
    }

    my $User = _checkUser($Servlet, $aUsers->[0], $Email);

    # Check if mailtype is activated.
    my %MailParams;
    my $MailType;
    if ($hFormValues->{'UseSystemMailType'}) {
        my $System   = LoadRootObject();
        $MailType = $System->folder('MailTypes')->child('NewPassword');
        $MailParams{'FROM'} = $Email;
    } else {
        $MailType = $Site->folder('MailTypes')->child('NewPassword');
    }
    return unless $MailType->get('IsActive');

    # Send mail.
    if (SendNewPasswordToken($Site, $User, $MailType, \%MailParams)) {
        $Servlet->params('PASSWORD_SEND_OK', 1);
    }
    else {
        $Servlet->params('SEND_MAIL_FAILED', 1);
    }

    return;
}

sub _checkUser {
    my ($Servlet,$User , $EMail) = @_;

    # Check if user is activated.
    $Servlet->form->executeFormError({
        'Reason'=> 'LOGIN_INACTIVE',
        'Form'  => 'NewPassword',
        'Name'  => 'Login',
        'Value' => $EMail,
    }) unless $User->get('IsActivated');

    return $User;
}

sub ViewUserLostPassword {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->redirectToSecureConnection($Servlet->object->getSite->get('WebUrlAdminSSL'));
}

1;
