package DE_EPAGES::Object::t::GetUnityURL;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Attributes::Object_Ext;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

sub test {

    note "Test Unity Storefront URLs";

    my $SubdomainSystem = _createMockedSystem(1, 1, 1);
    my $NoSubdomainSystem = _createMockedSystem(1, 1, 0);
    my $ShopWithDomainSSL = _createMockedShop(1, 1);
    my $ShopWithDomainNoSSL = _createMockedShop(1, 0);
    my $ShopWithoutDomain = _createMockedShop(0, 0);
    ##no critic (Subroutines::ProtectPrivateSubs)
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithoutDomain, $SubdomainSystem, 'UnityUrlScheme', 0),
        'http://testshop.hostname.de', 'Domain: no,  SSL: no,  Subdomain: yes');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithoutDomain, $SubdomainSystem, 'UnityUrlScheme', 1),
        'https://testshop.hostname.de', 'Domain: no,  SSL: yes, Subdomain: yes');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithoutDomain, $NoSubdomainSystem, 'UnityUrlScheme', 0),
        'http://hostname.de?shop=testshop', 'Domain: no,  SSL: no,  Subdomain: no');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithoutDomain, $NoSubdomainSystem, 'UnityUrlScheme', 1),
        'https://hostname.de?shop=testshop', 'Domain: no,  SSL: yes, Subdomain: no');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainNoSSL, $SubdomainSystem, 'UnityUrlScheme', 0),
        'http://testshop.hostname.de', 'Domain: yes, SSL: no,  Subdomain: yes');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, $SubdomainSystem, 'UnityUrlScheme', 1),
        'https://domain.de', 'Domain: yes, SSL: yes, Subdomain: yes');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainNoSSL, $SubdomainSystem, 'UnityUrlScheme', 1),
        'https://testshop.hostname.de', 'Domain: yes, SSL: yes, Subdomain: yes, no own certificate');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainNoSSL, $NoSubdomainSystem, 'UnityUrlScheme', 0),
        'http://hostname.de?shop=testshop', 'Domain: yes, SSL: no,  Subdomain: no');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, $NoSubdomainSystem, 'UnityUrlScheme', 1),
        'https://domain.de', 'Domain: yes, SSL: yes, Subdomain: no');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainNoSSL, $NoSubdomainSystem, 'UnityUrlScheme', 1),
        'https://hostname.de?shop=testshop', 'Domain: yes, SSL: yes, Subdomain: no, no own certificate');

    # test API URLs
    note('test API URLs');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, $SubdomainSystem, 'UnityApiUrlScheme', 0),
        'http://api.hostname.de/api/v2?shop=testshop', 'SSL: yes, API SSL: yes, no SSL request');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, $SubdomainSystem, 'UnityApiUrlScheme', 1),
        'https://api.hostname.de/api/v2?shop=testshop', 'SSL: yes, API SSL: yes (Domain: yes, Subdomain: yes)');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, $NoSubdomainSystem, 'UnityApiUrlScheme', 1),
        'https://api.hostname.de/api/v2?shop=testshop', 'SSL: yes, API SSL: yes (Domain: yes, Subdomain: no)');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithoutDomain, $SubdomainSystem, 'UnityApiUrlScheme', 1),
        'https://api.hostname.de/api/v2?shop=testshop', 'SSL: yes, API SSL: yes (Domain: no, Subdomain: yes)');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithoutDomain, $NoSubdomainSystem, 'UnityApiUrlScheme', 1),
        'https://api.hostname.de/api/v2?shop=testshop', 'SSL: yes, API SSL: yes (Domain: no, Subdomain: no)');

    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, _createMockedSystem(0, 1, 1), 'UnityApiUrlScheme', 1),
        'http://api.hostname.de/api/v2?shop=testshop', 'SSL: no, API SSL: yes');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, _createMockedSystem(1, 0, 1), 'UnityApiUrlScheme', 1),
        'http://api.hostname.de/api/v2?shop=testshop', 'SSL: yes, API SSL: no');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, _createMockedSystem(0, 0, 1), 'UnityApiUrlScheme', 1),
        'http://api.hostname.de/api/v2?shop=testshop', 'SSL: no, API SSL: no');

    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, _createMockedSystem(0, 1, 1, 1), 'UnityApiUrlScheme', 1),
        'http://api.hostname.de/api/v2?shop=testshop', 'With Api route in scheme: SSL: no, API SSL: yes');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, _createMockedSystem(1, 0, 1, 1), 'UnityApiUrlScheme', 1),
        'http://api.hostname.de/api/v2?shop=testshop', 'With Api route in scheme: SSL: yes, API SSL: no');
    is ( DE_EPAGES::Object::API::Attributes::Object_Ext::_GetUnityURL($ShopWithDomainSSL, _createMockedSystem(0, 0, 1, 1), 'UnityApiUrlScheme', 1),
        'http://api.hostname.de/api/v2?shop=testshop', 'With Api route in scheme: SSL: no, API SSL: no');

    done_testing();

    return;
}


sub _createMockedSystem {
    my($HasSSLCertificateUnity, $HasSSLCertificateUnityAPI, $UnityUseSubdomainRouting, $IsApiRouteInScheme) = @_;
    my $UnityApiUrlScheme = 'api.hostname.de';
    $UnityApiUrlScheme = $UnityApiUrlScheme . '/api/v2' if $IsApiRouteInScheme;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site', {
            'UnityUrlScheme'           => 'hostname.de',
            'UnityApiUrlScheme'        => $UnityApiUrlScheme,
            'UnityApiRoute'            => '/api/v2',
            'HasSSLCertificate'        => 1,
            'HasSSLCertificateUnity'   => $HasSSLCertificateUnity,
            'HasSSLCertificateUnityAPI' => $HasSSLCertificateUnityAPI,
            'UnityWebServerPort'       => undef,
            'UnitySSLWebServerPort'    => undef,
            'UnityUseSubdomainRouting' => $UnityUseSubdomainRouting
        })->getMockObject();
}

sub _createMockedShop {
    my ($WithDomain, $SSL) = @_;

    my $Shop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', {
            'DomainName'              => $WithDomain ? 'domain.de' : undef,
            'IsUnityRegistered'       => 1,
            'HasSSLCertificate'       => $SSL,
            'WebServerScriptNamePart' => 'testshop',
            'instanceOf' => 'Shop'
        })->getMockObject();


    $Shop->set({'Site' => $Shop});

    return $Shop;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&test );
});
1;
