#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::GetMerchantStatus_Service
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for GetMerchantStatus::Service.pm
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::GetMerchantStatus_Service;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::PayPalPlus::API::Webservice::Base::CredentialDTO;
use DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::RequestDTO;
use DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;

sub test {
    my $self = shift;
    $self->test_successful_request();
    $self->test_not_successful_request();
    $self->test_Repository_buildRequest_error();
    $self->test_Repository_persistance_error();
    $self->test_AvailablePayment_all_active();
    $self->test_AvailablePayment_all_inactive();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_successful_request {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GetMerchantStatusService = $self->_buildGetMerchantStatusService();
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus();
    my $ResultOfDoRequest = $GetMerchantStatusService->doRequest($PaymentMethodPayPalPlus);

    ok($ResultOfDoRequest,"$SubTestName - tests if the request was fine");

    is($GetMerchantStatusService->getErrorCode(), undef, "$SubTestName - Test if there is no error code");
    is($GetMerchantStatusService->getErrorMessage(), undef, "$SubTestName - Test if there is no error messages");
    is($GetMerchantStatusService->getMerchantId(), 'MerchantIdCodeValue', "$SubTestName - Test merchant id is returned");
    is($GetMerchantStatusService->getPrimaryEmail(), 'test.email@paypal.com', "$SubTestName - Test primary email is returned");
    is_deeply($GetMerchantStatusService->getProducts(), [{'name' => 'EXPRESS_CHECKOUT'}], "$SubTestName - Test products list is returned");
    is($GetMerchantStatusService->getMerchantClientId($PaymentMethodPayPalPlus), 'MerchantClientIdCodeValue', "$SubTestName - Test merchant client id is returned");
    ok(!$GetMerchantStatusService->hasMerchantStatusConflicts(), "$SubTestName - Test if there is no status conflict");
}
#----------------------------------------------------------------------------------------
sub test_not_successful_request {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GetMerchantStatusService = $self->_buildGetMerchantStatusService_notSuccessfulRestRequest();
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus();
    my $ResultOfDoRequest = $GetMerchantStatusService->doRequest($PaymentMethodPayPalPlus);

    is($ResultOfDoRequest, 0,"$SubTestName - tests if the request was not fine");

    is($GetMerchantStatusService->getErrorCode(), 'DummyErrorcode', "$SubTestName - Test if there is an error code");
    is($GetMerchantStatusService->getErrorMessage(), 'DummyErrormessage', "$SubTestName - Test if there is an error messages");
}
#----------------------------------------------------------------------------------------
sub test_Repository_buildRequest_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GetMerchantStatusService = $self->_buildGetMerchantStatusService_buildRequest_error();
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus();
    my $ResultOfDoRequest = $GetMerchantStatusService->doRequest($PaymentMethodPayPalPlus);

    is($ResultOfDoRequest,0,"$SubTestName - tests if the request was not fine");

    is($GetMerchantStatusService->getErrorCode(), 'RequestError', "$SubTestName - Test if there is an error code");
    is($GetMerchantStatusService->getErrorMessage(),
       'It was not possible to get the data for the Request from the database',
       "$SubTestName - Test if there is an error messages"
    );
}
#----------------------------------------------------------------------------------------
sub test_Repository_persistance_error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GetMerchantStatusService = $self->_buildGetMerchantStatusService_persistance_error();
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus();
    my $ResultOfDoRequest = $GetMerchantStatusService->doRequest($PaymentMethodPayPalPlus);
    is($ResultOfDoRequest,1,"$SubTestName - tests that request was done");
    my $ResultOfPersistResponse = $GetMerchantStatusService->persistResponse($PaymentMethodPayPalPlus);
    is($ResultOfPersistResponse,0,"$SubTestName - tests if the request was not fine");

    is($GetMerchantStatusService->getErrorCode(), 'PersistanceError', "$SubTestName - Test if there is an error code");
    is($GetMerchantStatusService->getErrorMessage(),
       'It was not possible to store the request in the database',
       "$SubTestName - Test if there is an error messages"
    );
}
#----------------------------------------------------------------------------------------
sub test_AvailablePayment_all_active {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GetMerchantStatusService = $self->_buildGetMerchantStatusService_all_active();
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus();
    my $ResultOfDoRequest = $GetMerchantStatusService->doRequest($PaymentMethodPayPalPlus);

    ok($ResultOfDoRequest,"$SubTestName - tests if the request was fine");

    is($GetMerchantStatusService->getErrorCode(), undef, "$SubTestName - Test if there is no error code");
    is($GetMerchantStatusService->getErrorMessage(), undef, "$SubTestName - Test if there is no error messages");
    is($GetMerchantStatusService->getMerchantId(), 'MerchantIdCodeValue', "$SubTestName - Test merchant id is returned");
    is($GetMerchantStatusService->getPrimaryEmail(), 'test.email@paypal.com', "$SubTestName - Test primary email is returned");
    is($GetMerchantStatusService->getIsActivePayPal(), 1, "$SubTestName - Test getIsActivePayPal is 1");
    is($GetMerchantStatusService->getIsActiveDirectDebit(), 1, "$SubTestName - Test getIsActiveDirectDebit is 1");
    is($GetMerchantStatusService->getIsActiveCreditCard(), 1, "$SubTestName - Test getIsActiveCreditCard is 1");
    is($GetMerchantStatusService->getIsActiveInvoice(), 1, "$SubTestName - Test getIsActiveInvoice is 1");
    is($GetMerchantStatusService->getMerchantClientId($PaymentMethodPayPalPlus), 'MerchantClientIdCodeValue', "$SubTestName - Test merchant client id is returned");
    ok(!$GetMerchantStatusService->hasMerchantStatusConflicts(), "$SubTestName - Test if there is no status conflict");
}
#----------------------------------------------------------------------------------------
sub test_AvailablePayment_all_inactive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GetMerchantStatusService = $self->_buildGetMerchantStatusService_all_inactive();
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus();
    my $ResultOfDoRequest = $GetMerchantStatusService->doRequest($PaymentMethodPayPalPlus);

    ok($ResultOfDoRequest,"$SubTestName - tests if the request was fine");

    is($GetMerchantStatusService->getErrorCode(), undef, "$SubTestName - Test if there is no error code");
    is($GetMerchantStatusService->getErrorMessage(), undef, "$SubTestName - Test if there is no error messages");
    is($GetMerchantStatusService->getMerchantId(), 'MerchantIdCodeValue', "$SubTestName - Test merchant id is returned");
    is($GetMerchantStatusService->getPrimaryEmail(), 'test.email@paypal.com', "$SubTestName - Test primary email is returned");
    is($GetMerchantStatusService->getIsActivePayPal(), 0, "$SubTestName - Test getIsActivePayPal is 0");
    is($GetMerchantStatusService->getIsActiveDirectDebit(), 0, "$SubTestName - Test getIsActiveDirectDebit is 0");
    is($GetMerchantStatusService->getIsActiveCreditCard(), 0, "$SubTestName - Test getIsActiveCreditCard is 0");
    is($GetMerchantStatusService->getIsActiveInvoice(), 0, "$SubTestName - Test getIsActiveInvoice is 0");
    is($GetMerchantStatusService->getMerchantClientId($PaymentMethodPayPalPlus), 'MerchantClientIdCodeValue', "$SubTestName - Test merchant client id is returned");
    ok(!$GetMerchantStatusService->hasMerchantStatusConflicts(), "$SubTestName - Test if there is no status conflict");
}

#----------------------------------------------------------------------------------------
#  helpers
#----------------------------------------------------------------------------------------
sub _buildGetMerchantStatusService {
    my $self = shift;
    my $hReturnValues = {
        '_createRepository' => $self->_createRepository(),
        '_createRestURLService' => $self->_createRestURLService(),
        '_createRestService' => $self->_buildRestService(),
    };

    return $self->_createGetMerchantStatusService($hReturnValues);
}
#----------------------------------------------------------------------------------------
sub _buildGetMerchantStatusService_all_active {
    my $self = shift;
    my $hReturnValues = {
        '_createRepository' => $self->_createRepository(),
        '_createRestURLService' => $self->_createRestURLService(),
        '_createRestService' => $self->_buildRestService_all_active(),
    };

    return $self->_createGetMerchantStatusService($hReturnValues);
}
#----------------------------------------------------------------------------------------
sub _buildGetMerchantStatusService_all_inactive {
    my $self = shift;
    my $hReturnValues = {
        '_createRepository' => $self->_createRepository(),
        '_createRestURLService' => $self->_createRestURLService(),
        '_createRestService' => $self->_buildRestService_all_inactive(),
    };

    return $self->_createGetMerchantStatusService($hReturnValues);
}
#----------------------------------------------------------------------------------------
sub _buildGetMerchantStatusService_notSuccessfulRestRequest {
    my $self = shift;
    my $hReturnValues = {
        '_createRepository' => $self->_createRepository(),
        '_createRestURLService' => $self->_createRestURLService(),
        '_createRestService' => $self->_buildRestService_notSuccessful(),
    };

    return $self->_createGetMerchantStatusService($hReturnValues);
}
#----------------------------------------------------------------------------------------
sub _buildGetMerchantStatusService_buildRequest_error {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;
    my $hReturnValues = {
        '_createRepository' => $self->_createRepository_with_buildRequestError(),
    };

    return $self->_createGetMerchantStatusService($hReturnValues);
}
#----------------------------------------------------------------------------------------
sub _buildGetMerchantStatusService_persistance_error {
    my $self = shift;
    my $hReturnValues = {
        '_createRepository' => $self->_createRepository_with_persistance_error(),
        '_createRestURLService' => $self->_createRestURLService(),
        '_createRestService' => $self->_buildRestService(),
    };

    return $self->_createGetMerchantStatusService($hReturnValues);
}
#----------------------------------------------------------------------------------------
sub _createGetMerchantStatusService {
    my $self = shift;
    my ($hReturnValues) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service', []
    );
    $Mockify->addMockWithReturnValue('_createRepository',$hReturnValues->{'_createRepository'});
    $Mockify->addMockWithReturnValue('_createRestURLService',$hReturnValues->{'_createRestURLService'});
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_createRestService',
        $hReturnValues->{'_createRestService'},
        [{'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRepository {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository', []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'buildRequest',
        undef,
        [{'object'=>'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]
    );
    $Mockify->addMockWithReturnValue('getRequestDTO', $self->_createRequestDTO());
    $Mockify->addMockWithReturnValue('getCredentialDTO', $self->_createCredentialDTO());
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'writeResponse',
        undef,
        [
            {'object' => 'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service'},
            {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}
        ]
    );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRepository_with_buildRequestError {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository', []
    );
    $Mockify->addMock( 'buildRequest', sub {
        Error('PARAMETER_MISSING');
    } );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRepository_with_persistance_error {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository', []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'buildRequest',
        undef,
        [{'object'=>'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]
    );
    $Mockify->addMockWithReturnValue('getRequestDTO', $self->_createRequestDTO());
    $Mockify->addMockWithReturnValue('getCredentialDTO', $self->_createCredentialDTO());
    $Mockify->addMock( 'writeResponse', sub {
        Error('PARAMETER_MISSING');
    } );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRestURLService {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::RestURLService',
                   []
              );
    $Mockify->addMockWithReturnValueAndParameterCheck('buildEpagesJRestUrl', 'http://myDummyURL.info', [
        {'string' => 'payments/paypalplus/merchantstatus'},
        {'hashref' => {} },
    ]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildRestService {
    my $self = shift;
    my $ReturnValues = {
        'post' => 1,
        'getResponse' => {
            'merchant_id'=>'MerchantIdCodeValue',
            'tracking_id' => 'TrackingIdCodeValue',
            'primary_email' => 'test.email@paypal.com',
            'products' => [
                {
                    'name' => 'EXPRESS_CHECKOUT'
                }
            ],
            'payments_receivable' => 'true',
            'primary_email_confirmed' => 'true',
            'oauth_integrations' => [
                {
                    'oauth_third_party' => [
                        {
                            'partner_client_id' => 'PartnerClientIdCodeValue',
                            'merchant_client_id' => 'MerchantClientIdCodeValue'
                        }
                    ]
                }
            ]
        },
    };

    return $self->_createRestService($ReturnValues);
}

#----------------------------------------------------------------------------------------
sub _buildRestService_all_active {
    my $self = shift;
    my $ReturnValues = {
        'post' => 1,
        'getResponse' => {
            'merchant_id'=>'MerchantIdCodeValue',
            'tracking_id' => 'TrackingIdCodeValue',
            'primary_email' => 'test.email@paypal.com',
            'products' => [
                {'name' => 'BASIC_PPPLUS_CORE','status'=>'ACTIVE'},
                {'name' => 'BASIC_PPPLUS_GUEST_ELV','status'=>'ACTIVE'},
                {'name' => 'BASIC_PPPLUS_GUEST_CC','status'=>'ACTIVE'},
                {'name' => 'BASIC_PPPLUS_PUI','status'=>'ACTIVE'}
            ],
            'payments_receivable' => 'true',
            'primary_email_confirmed' => 'true',
            'oauth_integrations' => [
                {
                    'oauth_third_party' => [
                        {
                            'partner_client_id' => 'PartnerClientIdCodeValue',
                            'merchant_client_id' => 'MerchantClientIdCodeValue'
                        }
                    ]
                }
            ]
        },
    };

    return $self->_createRestService($ReturnValues);
}
#----------------------------------------------------------------------------------------
sub _buildRestService_all_inactive {
    my $self = shift;
    my $ReturnValues = {
        'post' => 1,
        'getResponse' => {
            'merchant_id'=>'MerchantIdCodeValue',
            'tracking_id' => 'TrackingIdCodeValue',
            'primary_email' => 'test.email@paypal.com',
            'products' => [
                {'name' => 'BASIC_PPPLUS_CORE','status'=>'INACTIVE'},
                {'name' => 'BASIC_PPPLUS_GUEST_ELV','status'=>'INACTIVE'},
                {'name' => 'BASIC_PPPLUS_GUEST_CC','status'=>'INACTIVE'},
                {'name' => 'BASIC_PPPLUS_PUI','status'=>'INACTIVE'}
            ],
            'payments_receivable' => 'true',
            'primary_email_confirmed' => 'true',
            'oauth_integrations' => [
                {
                    'oauth_third_party' => [
                        {
                            'partner_client_id' => 'PartnerClientIdCodeValue',
                            'merchant_client_id' => 'MerchantClientIdCodeValue'
                        }
                    ]
                }
            ]
        },
    };

    return $self->_createRestService($ReturnValues);
}
#----------------------------------------------------------------------------------------
sub _buildRestService_notSuccessful {
    my $self = shift;
    my $ReturnValues = {
        'post' => 0,
        'getErrorCode' => 'DummyErrorcode',
        'getErrorMessage'=> 'DummyErrormessage',
    };

    return $self->_createRestService($ReturnValues);
}
#----------------------------------------------------------------------------------------
sub _createRestService {
    my $self = shift;
    my ($ReturnValues) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::API::Webservice::RestService', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('post', $ReturnValues->{'post'}, [
        {'string' => 'http://myDummyURL.info'},
        {'hashref' => {
            'merchantClientId' => undef,
            'providerClientSecret' => 'MyDummyProviderClientSecret',
            'providerClientId' => 'MyDummyProviderClientID',
            'payPalPartnerAttributionId' => 'myAttributionId',
            'isTestTransaction' => 'true',
        }},
        {'hashref' => {
          'providerPayPalAccountId' => 'DummyProviderPayPalAccountId',
          'trackingId' => '1234565789JAJAGENAU',
          'merchantId' => 'DummyMerchantId'
        }},
        {int => 1},
    ]);
    $Mockify->addMockWithReturnValue('getResponse', $ReturnValues->{'getResponse'});
    $Mockify->addMockWithReturnValue('getErrorCode', $ReturnValues->{'getErrorCode'});
    $Mockify->addMockWithReturnValue('getErrorMessage', $ReturnValues->{'getErrorMessage'});
    $Mockify->addMockWithReturnValueAndParameterCheck('setLogErrorsInComunicationlog', undef, [{'int' => 1}]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethodPayPalPlus {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus',
        $hAttributes
    );
    $Mockify->addMockWithReturnValue('getPartnerClientId', 'PartnerClientIdCodeValue');

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRequestDTO {
    my $self = shift;

    my $RequestDTO = DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::RequestDTO->new();
    $RequestDTO->trackingId('1234565789JAJAGENAU');
    $RequestDTO->providerPayPalAccountId('DummyProviderPayPalAccountId');
    $RequestDTO->merchantId('DummyMerchantId');
    return $RequestDTO;
}
#----------------------------------------------------------------------------------------
sub _createCredentialDTO {
    my $self = shift;
    my $CredentialDTO = DE_EPAGES::PayPalPlus::API::Webservice::Base::CredentialDTO->new();
    $CredentialDTO->isInTestMode(1);
    $CredentialDTO->providerClientSecret('MyDummyProviderClientSecret');
    $CredentialDTO->providerClientId('MyDummyProviderClientID');
    $CredentialDTO->payPalPartnerAttributionId('myAttributionId');
    return $CredentialDTO;
}
__PACKAGE__->runScript();