#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::UI_PaymentMethodPayPalPlus
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::PaymentMethodPayPalPlus
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::UI_PaymentMethodPayPalPlus;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use Test::Exception;
use Test::More;
use Test::Mockify;
use Test::Mockify::Matcher qw (Object String HashRef);
use strict;

sub test {
    my $self = shift;

    $self->test_ViewSettings_NoSslCertificate_ShopNotValidatedWithPayPalPlus();
    $self->test_ViewSettings_ShopAlreadyValidatedWithPayPalPlus();
    $self->test_ViewSettings_MerchantCredentialsInDatabase();
    $self->test_ViewSettings_MerchantStatusWithConflicts();
    $self->test_ViewSettings_MerchantStatusAuthError_OnboardingUrlError();
    $self->test_SaveSettings();
    $self->test_DisablePayPalPlusConnectivity();
    $self->test_DisablePayPalPlusConnectivity_IsDefaultPayment();
    $self->test_ViewSettings_MerchantStatusAvailablePayments_AllActive();
    $self->test_ViewSettings_MerchantStatusAvailablePayments_AllInactive();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_NoSslCertificate_ShopNotValidatedWithPayPalPlus {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '0';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $Logger = $self->_createLogger();
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus_BuildMerchantStatusNotFoundError($System, $Logger);
    my $HasCollectedMerchantStatus = 0;
    my $Servlet = $self->_build_Servlet_UndefinedMerchantCredentials($HasCollectedMerchantStatus);

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if 'ViewSettings' does not die");

    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_fetchOnBoardingUrlOnServlet'),
            "$SubTestName - method '_fetchOnBoardingUrlOnServlet' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_gatherMerchantStatusFromService'),
            "$SubTestName - method '_gatherMerchantStatusFromService' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_isMerchantConnected'),
            "$SubTestName - method '_isMerchantConnected' was called");
    ok(!WasCalled($UI_PaymentMethodPayPalPlus, '_gatherExperienceProfileIdFromDatabaseOrService'),
            "$SubTestName - method '_gatherExperienceProfileIdFromDatabaseOrService' was NOT called");
    ok(!WasCalled($UI_PaymentMethodPayPalPlus, '_createExperienceProfileIdService'),
            "$SubTestName - method '_createExperienceProfileIdService' was NOT called");

    my $aParamsFromCall0 = GetParametersFromMockifyCall($Servlet, 'vars', 0);
    my $aParamsFromCall1 = GetParametersFromMockifyCall($Servlet, 'vars', 1);
    my $aParamsFromCall2 = GetParametersFromMockifyCall($Servlet, 'vars', 2);
    is_deeply($aParamsFromCall0, ['NoValidSSLCertificateError', '1'], "$SubTestName - tests if error was set to Servlet");
    is_deeply($aParamsFromCall1, ['OnboardingURL', 'https://du.mmy.url/path?p=1&p=2'], "$SubTestName - tests OnboardingURL was set to Servlet");
    is_deeply($aParamsFromCall2, ['HasCollectedMerchantStatus', '0'], "$SubTestName - tests no merchant data collected was set to Servlet");

    my $aParamsFromLoggerCall0 = GetParametersFromMockifyCall($Logger, 'logOnboarding', 0);
    like($aParamsFromLoggerCall0->[0]->{'Status'}, qr/Merchant not connected yet/, "$SubTestName - prove 'Merchant not connected yet'  status");
}

#----------------------------------------------------------------------------------------
sub test_ViewSettings_ShopAlreadyValidatedWithPayPalPlus {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $Logger = $self->_createLogger();
    my $HasMerchantStatusConflicts = 0;
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus($System, $Logger, $HasMerchantStatusConflicts);
    my $HasCollectedMerchantStatus = 1;
    my $Servlet = $self->_build_Servlet_UndefinedMerchantCredentials($HasCollectedMerchantStatus);

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if 'ViewSettings' does not die");

    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_fetchOnBoardingUrlOnServlet'),
            "$SubTestName - method '_fetchOnBoardingUrlOnServlet' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_gatherMerchantStatusFromService'),
            "$SubTestName - method '_gatherMerchantStatusFromService' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_isMerchantConnected'),
            "$SubTestName - method '_isMerchantConnected' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_gatherExperienceProfileIdFromDatabaseOrService'),
            "$SubTestName - method '_gatherExperienceProfileIdFromDatabaseOrService' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_createExperienceProfileIdService'),
            "$SubTestName - method '_createExperienceProfileIdService' was called");

    my $aParamsFromCall0 = GetParametersFromMockifyCall($Servlet, 'vars', 0);
    is_deeply($aParamsFromCall0, ['OnboardingURL', 'https://du.mmy.url/path?p=1&p=2'], "$SubTestName - tests OnboardingURL was set to Servlet");

    my $aParamsFromCall1 = GetParametersFromMockifyCall($Servlet, 'vars', 1);
    is_deeply($aParamsFromCall1, ['HasCollectedMerchantStatus', '1'], "$SubTestName - tests no merchant data collected was set to Servlet");

    my $aParamsFromCall2 = GetParametersFromMockifyCall($Servlet, 'vars', 2);
    is_deeply($aParamsFromCall2, ['HasMerchantStatusConflicts', '0'], "$SubTestName - tests merchant status data has NO conflicts");

    my $aParamsFromCall3 = GetParametersFromMockifyCall($Servlet, 'vars', 3);
    is_deeply($aParamsFromCall3, ['GetMerchantStatusPrimaryEmail', 'test.mail@epages.com'], "$SubTestName - tests merchant status data is returned");

    my $aParamsFromLoggerCall0 = GetParametersFromMockifyCall($Logger, 'logOnboarding', 0);
    like($aParamsFromLoggerCall0->[0]->{'Status'}, qr/Add MerchantId & MerchantClientId to PaymentMethod/, "$SubTestName - prove 'Add MerchantId & MerchantClientId to PaymentMethod'  status");

}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_MerchantCredentialsInDatabase {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $Logger = $self->_createLogger();
    my $HasMerchantStatusConflicts = 0;
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus($System, $Logger, $HasMerchantStatusConflicts);
    my $HasCollectedMerchantStatus = 1;
    my $Servlet = $self->_build_Servlet_DefinedMerchantCredentials($HasCollectedMerchantStatus);

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if 'ViewSettings' does not die");

    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_fetchOnBoardingUrlOnServlet'),
            "$SubTestName - method '_fetchOnBoardingUrlOnServlet' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_gatherMerchantStatusFromService'),
            "$SubTestName - method '_gatherMerchantStatusFromService' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_isMerchantConnected'),
            "$SubTestName - method '_isMerchantConnected' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_gatherExperienceProfileIdFromDatabaseOrService'),
            "$SubTestName - method '_gatherExperienceProfileIdFromDatabaseOrService' was called");
    ok(!WasCalled($UI_PaymentMethodPayPalPlus, '_createExperienceProfileIdService'),
            "$SubTestName - method '_createExperienceProfileIdService' was NOT called");

    my $aParamsFromCall3 = GetParametersFromMockifyCall($Servlet, 'vars', 3);
    is_deeply($aParamsFromCall3, ['GetMerchantStatusPrimaryEmail', 'test.mail@epages.com'], "$SubTestName - tests merchant status data is returned");

    my $aParamsFromLoggerCall0 = GetParametersFromMockifyCall($Logger, 'logOnboarding', 0);
    like($aParamsFromLoggerCall0->[0]->{'Status'}, qr/Add MerchantId & MerchantClientId to PaymentMethod/, "$SubTestName - prove 'Add MerchantId & MerchantClientId to PaymentMethod'  status");

}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_MerchantStatusWithConflicts {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $Logger = $self->_createLogger();
    my $HasMerchantStatusConflicts = 1;
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus($System, $Logger, $HasMerchantStatusConflicts);
    my $HasCollectedMerchantStatus = 1;
    my $Servlet = $self->_build_Servlet_DefinedMerchantCredentials($HasCollectedMerchantStatus);

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if 'ViewSettings' does not die");

    my $aParamsFromCall2 = GetParametersFromMockifyCall($Servlet, 'vars', 2);
    is_deeply($aParamsFromCall2, ['HasMerchantStatusConflicts', 1], "$SubTestName - tests merchant status data has conflicts");

    my $aParamsFromCall3 = GetParametersFromMockifyCall($Servlet, 'vars', 3);
    is_deeply($aParamsFromCall3,
            [ 'GetMerchantStatusConflicts', [ {'Conflict' => 'PaymentsReceivable' }, {'Conflict' => 'PrimaryEmailConfirmed'} ] ],
            "$SubTestName - tests merchant status data has conflicts");
}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_MerchantStatusAuthError_OnboardingUrlError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $Logger = $self->_createLogger();
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus_BuildMerchantStatusAuthError_BuildOnboardingUrlError($System, $Logger);
    my $HasCollectedMerchantStatus = 0;
    my $Servlet = $self->_build_Servlet_DefinedMerchantCredentials($HasCollectedMerchantStatus);

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if 'ViewSettings' does not die");

    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_fetchOnBoardingUrlOnServlet'),
            "$SubTestName - method '_fetchOnBoardingUrlOnServlet' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_gatherMerchantStatusFromService'),
            "$SubTestName - method '_gatherMerchantStatusFromService' was called");
    ok(WasCalled($UI_PaymentMethodPayPalPlus, '_isMerchantConnected'),
            "$SubTestName - method '_isMerchantConnected' was called");
    ok(!WasCalled($UI_PaymentMethodPayPalPlus, '_gatherExperienceProfileIdFromDatabaseOrService'),
            "$SubTestName - method '_gatherExperienceProfileIdFromDatabaseOrService' was NOT called");
    ok(!WasCalled($UI_PaymentMethodPayPalPlus, '_createExperienceProfileIdService'),
            "$SubTestName - method '_createExperienceProfileIdService' was NOT called");

    my $aParamsFromLoggerCall0 = GetParametersFromMockifyCall($Logger, 'logOnboarding', 0);
    is($aParamsFromLoggerCall0->[0]->{'Status'},'The Boarding Url could not be created.', "$SubTestName - prove 'boarding url couldn be created' Error status");
    like($aParamsFromLoggerCall0->[0]->{'ErrorMessage'}, qr/BuildUrlError/, "$SubTestName - prove 'BuildUrlError' Error type ");
    my $aParamsFromLoggerCall1 = GetParametersFromMockifyCall($Logger, 'logOnboarding', 1);
    like($aParamsFromLoggerCall1->[0]->{'Status'}, qr/Unexpected web service error when getting merchant status information.*401 Unauthorized/, "$SubTestName - prove merchant status unauthorized error status");

    my $aParamsFromServletCall0 = GetParametersFromMockifyCall($Servlet, 'vars', 0);
    is_deeply($aParamsFromServletCall0, ['OnboardingError', 1], "$SubTestName - tests onboarding URL has an error");
    my $aParamsFromServletCall1 = GetParametersFromMockifyCall($Servlet, 'vars', 1);
    is_deeply($aParamsFromServletCall1, ['HasCollectedMerchantStatus', 0], "$SubTestName - tests merchant status data has not been collected");
    my $aParamsFromServletCall2 = GetParametersFromMockifyCall($Servlet, 'vars', 2);
    is_deeply($aParamsFromServletCall2, ['GetMerchantStatusError', 1], "$SubTestName - tests merchant status has an error");
}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_MerchantStatusAvailablePayments_AllActive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $Logger = $self->_createLogger();
    my $HasMerchantStatusConflicts = 0;

    my $aPaypalPayments = [
        {'name' => 'BASIC_PPPLUS_CORE', 'status' => 'ACTIVE'},
        {'name' => 'BASIC_PPPLUS_GUEST_ELV', 'status' => 'ACTIVE'},
        {'name' => 'BASIC_PPPLUS_GUEST_CC', 'status' => 'ACTIVE'},
        {'name' => 'BASIC_PPPLUS_PUI', 'status' => 'ACTIVE'}
    ];

    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus_withAvailablePayments($System, $Logger, $HasMerchantStatusConflicts, $aPaypalPayments);
    my $HasCollectedMerchantStatus = 1;
    my $Servlet = $self->_build_Servlet_DefinedMerchantCredentials_noServletVarsMocking($HasCollectedMerchantStatus);

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if 'ViewSettings' does not die");

    is($Servlet->vars('IsActivePayPal'), 1, "$SubTestName - tests Servlet var IsActivePayPal is set to 1");
    is($Servlet->vars('IsActiveDirectDebit'), 1, "$SubTestName - tests Servlet var IsActiveDirectDebit is set to 1");
    is($Servlet->vars('IsActiveCreditCard'), 1, "$SubTestName - tests Servlet var IsActiveCreditCard is set to 1");
    is($Servlet->vars('IsActiveInvoice'), 1, "$SubTestName - tests Servlet var IsActiveInvoice is set to 1");

}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_MerchantStatusAvailablePayments_AllInactive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $Logger = $self->_createLogger();
    my $HasMerchantStatusConflicts = 0;

    my $aPaypalPayments = [
        {'name' => 'BASIC_PPPLUS_CORE', 'status' => 'INACTIVE'},
        {'name' => 'BASIC_PPPLUS_GUEST_ELV', 'status' => 'INACTIVE'},
        {'name' => 'BASIC_PPPLUS_GUEST_CC', 'status' => 'INACTIVE'},
        {'name' => 'BASIC_PPPLUS_PUI', 'status' => 'INACTIVE'}
    ];

    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus_withAvailablePayments($System, $Logger, $HasMerchantStatusConflicts, $aPaypalPayments);
    my $HasCollectedMerchantStatus = 1;
    my $Servlet = $self->_build_Servlet_DefinedMerchantCredentials_noServletVarsMocking($HasCollectedMerchantStatus);

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if 'ViewSettings' does not die");

    is($Servlet->vars('IsActivePayPal'), 0, "$SubTestName - tests Servlet var IsActivePayPal is set to 0");
    is($Servlet->vars('IsActiveDirectDebit'), 0, "$SubTestName - tests Servlet var IsActiveDirectDebit is set to 0");
    is($Servlet->vars('IsActiveCreditCard'), 0, "$SubTestName - tests Servlet var IsActiveCreditCard is set to 0");
    is($Servlet->vars('IsActiveInvoice'), 0, "$SubTestName - tests Servlet var IsActiveInvoice is set to 0");

}
#----------------------------------------------------------------------------------------
sub test_SaveSettings {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus();
    my $Servlet = $self->_build_Servlet_UndefinedMerchantCredentials();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->SaveSettings($Servlet) }, "$SubTestName - tests if 'SaveSettings' does not die");
}
#----------------------------------------------------------------------------------------
sub test_DisablePayPalPlusConnectivity {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus();
    my $Servlet = $self->_build_Servlet_UndefinedMerchantCredentials();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->DisablePayPalPlusConnectivity($Servlet) }, "$SubTestName - tests if 'DisablePayPalPlusConnectivity' does not die");
}
#----------------------------------------------------------------------------------------
sub test_DisablePayPalPlusConnectivity_IsDefaultPayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus();
    my $Servlet = $self->_build_Servlet_IsDefaultPayment();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->DisablePayPalPlusConnectivity($Servlet) }, "$SubTestName - tests if 'DisablePayPalPlusConnectivity' does not die");

    my $aParamsFromCall0 = GetParametersFromMockifyCall($Servlet, 'vars', 0);
    is_deeply($aParamsFromCall0, ['IsDefaultPaymentMethodAndCanNotBeDisconnected', 1], "$SubTestName - servlet variable 'IsDefaultPaymentMethodAndCanNotBeDisconnected' is set");
}

#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($hIdCredentials, $IsDefaultPayment, $HasCollectedMerchantStatus) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Presentation::Servlet::Permission',
         []
    );
    $Mockify->addMockWithReturnValue('object', $self->_createPaymentMethodPayPalPlusWithAttributeValues($hIdCredentials, $IsDefaultPayment));
    $Mockify->addMockWithReturnValue('form', $self->_createForm());
    $Mockify->addMockWithReturnValueAndParameterCheck('vars', undef, ['string', 'any']);
    $Mockify->addMockWithReturnValueAndParameterCheck('params', $HasCollectedMerchantStatus, [{'string' => 'HasCollectedMerchantStatus'}]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createServlet_noVarsMocking {
    my $self = shift;
    my ($hIdCredentials, $IsDefaultPayment, $HasCollectedMerchantStatus) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Presentation::Servlet::Permission',
         []
    );
    $Mockify->addMockWithReturnValue('object', $self->_createPaymentMethodPayPalPlusWithAttributeValues($hIdCredentials, $IsDefaultPayment));
    $Mockify->addMockWithReturnValue('form', $self->_createForm());
    $Mockify->addMockWithReturnValueAndParameterCheck('params', $HasCollectedMerchantStatus, [{'string' => 'HasCollectedMerchantStatus'}]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Presentation::API::Form',
         []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'form',
        undef,
        [
            {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
            {'string' => 'PayPalPlusSettings'}
        ]
    );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'ID' => '23456',
        'Alias' => 'NemoShop',
        'OrderProcess' => 1,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );
    return $Mockify->getMockObject();;
}
#----------------------------------------------------------------------------------------
sub _createSystem {
    my $self = shift;
    my ($HasSSLCertificate) = @_;

    my $hAttributes = {
        'HasSSLCertificate' => $HasSSLCertificate
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createClass {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Object::API::Object::Class', {} );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethodPayPalPlusWithAttributeValues {
    my $self = shift;
    my ( $hIdCredentials, $IsDefaultPayment ) = @_;

    my $hAttributes = {
        'Alias' => 'PayPalPLUS',
        'ClientId' => 'ClientId_PaymentMethod',
        'ClientSecret' => 'ClientSecret_PaymentMethod',
        'ExperienceProfileId' => $hIdCredentials->{'ExperienceProfileId'},
        'ID' => '12345',
        'IsDefault' => $IsDefaultPayment // 0,
        'MerchantClientId' => $hIdCredentials->{'MerchantClientId'},
        'Path' => '/Shops/NemoShop/PaymentMethods/PayPalPlusTest',
        'Site' => $self->_createShop(),
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );

    my $Class = $self->_createClass();
    $Mockify->mock('class', sub {return $Class;});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _build_Servlet_UndefinedMerchantCredentials {
    my $self = shift;
    my ($HasCollectedMerchantStatus) = @_;

    my $hIdCredentials = {
        'ExperienceProfileId' => undef,
        'MerchantClientId' => undef,
        'MerchantId' => undef,
        'OnboardingTransactionId' => undef
    };
    my $IsDefaultPayment = 0;

    return $self->_createServlet($hIdCredentials, $IsDefaultPayment, $HasCollectedMerchantStatus);
}
#----------------------------------------------------------------------------------------
sub _build_Servlet_IsDefaultPayment {
    my $self = shift;

    my $hIdCredentials = {
        'MerchantClientId' => undef,
        'ExperienceProfileId' => undef
    };
    my $IsDefaultPayment = 1;
    my $HasCollectedMerchantStatus = 1;

    return $self->_createServlet($hIdCredentials, $IsDefaultPayment, $HasCollectedMerchantStatus);
}
#----------------------------------------------------------------------------------------
sub _build_Servlet_DefinedMerchantCredentials {
    my $self = shift;
    my ($HasCollectedMerchantStatus) = @_;

    my $hIdCredentials = {
        'MerchantClientId' => 'MerchantClientId',
        'MerchantId' => 'MerchantId',
        'ExperienceProfileId' => 'ExperienceProfileId',
        'OnboardingTransactionId' => 'OnboardingTransactionId'
    };
    my $IsDefaultPayment = 1;

    return $self->_createServlet($hIdCredentials, $IsDefaultPayment, $HasCollectedMerchantStatus);
}
#----------------------------------------------------------------------------------------
sub _build_Servlet_DefinedMerchantCredentials_noServletVarsMocking {
    my $self = shift;
    my ($HasCollectedMerchantStatus) = @_;

    my $hIdCredentials = {
        'MerchantClientId' => 'MerchantClientId',
        'MerchantId' => 'MerchantId',
        'ExperienceProfileId' => 'ExperienceProfileId',
        'OnboardingTransactionId' => 'OnboardingTransactionId'
    };
    my $IsDefaultPayment = 1;

    return $self->_createServlet_noVarsMocking($hIdCredentials, $IsDefaultPayment, $HasCollectedMerchantStatus);
}
#----------------------------------------------------------------------------------------
sub _build_UI_PaymentMethodPaypalPlus {
    my $self = shift;
    my ($System, $Logger, $HasMerchantStatusConflicts) = @_;

    my $hReturnValues = {
        '_createExperienceProfileIdService' => $self->_createExperienceProfileIdService(),
        'attributeValues' => {'Values' => {'ClientSecret' => 'ClientSecret', 'ClientId' => 'ClientId'}},
        '_createOnBoardingService' => $self->_createOnBoardingService(),
        '_createGetMerchantStatusService' => $self->_createGetMerchantStatusService($HasMerchantStatusConflicts),
        '_LoadRootObject' => $System,
        '_createLogger' => $Logger,

    };
    return $self->_create_UI_PaymentMethodPaypalPlus( $hReturnValues );
}
#----------------------------------------------------------------------------------------
sub _build_UI_PaymentMethodPaypalPlus_withAvailablePayments {
    my $self = shift;
    my ($System, $Logger, $HasMerchantStatusConflicts, $aProducts) = @_;

    my $hReturnValues = {
        '_createExperienceProfileIdService' => $self->_createExperienceProfileIdService(),
        'attributeValues' => {'Values' => {'ClientSecret' => 'ClientSecret', 'ClientId' => 'ClientId'}},
        '_createOnBoardingService' => $self->_createOnBoardingService(),
        '_createGetMerchantStatusService' => $self->_createGetMerchantStatusService_withAvailablePayments($HasMerchantStatusConflicts, $aProducts),
        '_LoadRootObject' => $System,
        '_createLogger' => $Logger,
    };
    return $self->_create_UI_PaymentMethodPaypalPlus( $hReturnValues );
}

#----------------------------------------------------------------------------------------
sub _build_UI_PaymentMethodPaypalPlus_BuildMerchantStatusNotFoundError {
    my $self = shift;
    my ($System, $Logger) = @_;

    my $hReturnValues = {
        '_createExperienceProfileIdService' => $self->_createExperienceProfileIdService(),
        'attributeValues' => {'Values' => {'ClientSecret' => 'ClientSecret', 'ClientId' => 'ClientId'}},
        '_createOnBoardingService' => $self->_createOnBoardingService(),
        '_createGetMerchantStatusService' => $self->_createGetMerchantStatusService_withError('404 Not Found'),
        '_LoadRootObject' => $System,
        '_createLogger' => $Logger,
    };
    return $self->_create_UI_PaymentMethodPaypalPlus( $hReturnValues );
}

#----------------------------------------------------------------------------------------
sub _build_UI_PaymentMethodPaypalPlus_BuildMerchantStatusAuthError_BuildOnboardingUrlError {
    my $self = shift;
    my ($System, $Logger) = @_;

    my $hReturnValues = {
        '_createExperienceProfileIdService' => $self->_createExperienceProfileIdService(),
        'attributeValues' => {'Values' => {'ClientSecret' => 'ClientSecret', 'ClientId' => 'ClientId'}},
        '_createOnBoardingService' => $self->_createOnBoardingService_withError(),
        '_createGetMerchantStatusService' => $self->_createGetMerchantStatusService_withError('401 Unauthorized'),
        '_LoadRootObject' => $System,
        '_createLogger' => $Logger,
    };
    return $self->_create_UI_PaymentMethodPaypalPlus( $hReturnValues );
}

#----------------------------------------------------------------------------------------
sub _create_UI_PaymentMethodPaypalPlus {
    my $self = shift;
    my ($hReturnValues) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus',
         []
    );
    $Mockify->addMethodSpy('_isMerchantConnected');
    $Mockify->addMethodSpy('_gatherMerchantStatusFromService');
    $Mockify->addMethodSpy('_gatherExperienceProfileIdFromDatabaseOrService');
    $Mockify->addMethodSpy('_createExperienceProfileIdService');
    $Mockify->addMethodSpy('_fetchOnBoardingUrlOnServlet');
    $Mockify->addMockWithReturnValueAndParameterCheck('Save', undef, [{'object' => 'DE_EPAGES::Presentation::Servlet::Permission'}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('_createExperienceProfileIdService', $hReturnValues->{'_createExperienceProfileIdService'} , [{'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]);
    $Mockify->addMockWithReturnValue('_createGetMerchantStatusService', $hReturnValues->{'_createGetMerchantStatusService'});
    $Mockify->addMockWithReturnValueAndParameterCheck('_BuildShopUrl', 'ReturnURL', [
        {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
        {'hashref' => {'ViewAction' => 'MBO-ViewSettings'}},
        'hashref',
        {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'}
    ]);
    $Mockify->addMockWithReturnValue('_LoadRootObject', $hReturnValues->{'_LoadRootObject'});
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'attributeValues',
        $hReturnValues->{'attributeValues'},
        [
            {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'},
            {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
            {'object' => 'DE_EPAGES::Object::API::Object::Class'}
        ]
    );
    $Mockify->mock('_createLogger', $hReturnValues->{'_createLogger'}, []);
    $Mockify->mock('_createOnBoardingService',$hReturnValues->{'_createOnBoardingService'} );
    my $PaymentMethodPaypalPlus = $Mockify->getMockObject();
    return $PaymentMethodPaypalPlus;
}
#----------------------------------------------------------------------------------------
sub _createOnBoardingService {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::OnBoarding::Service',
                   $aParameterList
              );
    $Mockify->mock('getOnboardingURL')->when()->thenReturn('https://du.mmy.url/path?p=1&p=2');
    $Mockify->mock('doRequest')->when(Object(), String('ReturnURL'))->thenReturnUndef();

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createOnBoardingService_withError {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Webservice::OnBoarding::Service',
                   $aParameterList
              );
    $Mockify->mock('doRequest')->whenAny()->thenThrowError('BuildUrlError');

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLogger {
    my $self = shift;
    my ($LogData) = @_;
    my $aParameterList = [];
    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::Logger',
                   $aParameterList
              );
    $Mockify->mock('logOnboarding')->when(HashRef())->thenReturnUndef();
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createExperienceProfileIdService {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service',
         []
    );
    $Mockify->addMockWithReturnValue('doRequest', 1);
    $Mockify->addMockWithReturnValue('persistResponse', 1);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createGetMerchantStatusService {
    my $self = shift;
    my ($HasMerchantStatusConflicts) = @_;

    my $Mockify = Test::Mockify->new(
        'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service',
         []
    );
    $Mockify->mock('doRequest')->when(Object('DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'))->thenReturn(1);
    $Mockify->mock('persistResponse')->when(Object('DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'))->thenReturn(1);
    my $hGetResponseReturn = {
            'merchant_id' => 'Test_Merchant_Id',
            'payments_receivable' => $HasMerchantStatusConflicts ? 'false' : 'true',
            'primary_email_confirmed' => $HasMerchantStatusConflicts ? 'false' : 'true',
            'primary_email' => 'test.mail@epages.com'
    };
    $Mockify->mock('getResponse')->whenAny()->thenReturn($hGetResponseReturn);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createGetMerchantStatusService_withAvailablePayments {
    my $self = shift;
    my ($HasMerchantStatusConflicts, $aProducts) = @_;

    my $Mockify = Test::Mockify->new(
        'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service',
         []
    );
    $Mockify->mock('doRequest')->when(Object('DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'))->thenReturn(1);
    $Mockify->mock('persistResponse')->when(Object('DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'))->thenReturn(1);
    my $hGetResponseReturn = {
            'merchant_id' => 'Test_Merchant_Id',
            'payments_receivable' => $HasMerchantStatusConflicts ? 'false' : 'true',
            'primary_email_confirmed' => $HasMerchantStatusConflicts ? 'false' : 'true',
            'primary_email' => 'test.mail@epages.com'
    };
    $Mockify->mock('getResponse')->whenAny()->thenReturn($hGetResponseReturn);
    $Mockify->mock('getProducts')->when()->thenReturn($aProducts);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createGetMerchantStatusService_withError {
    my $self = shift;
    my ($ErrorMessage) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service',
         []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('doRequest', 0, [{'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('persistResponse', 1, [{'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]);
    $Mockify->addMockWithReturnValue('getResponse', { 'merchant_id' => undef });
    $Mockify->addMockWithReturnValue('getErrorCode', 'WebServiceError');
    $Mockify->addMockWithReturnValue('getErrorMessage', $ErrorMessage);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
