#========================================================================================
# package      DE_EPAGES::Presentation::DAL::SQL::Users
# state        private
# description  standard SQL database interface for license
#========================================================================================
package DE_EPAGES::Presentation::DAL::SQL::Users;

use strict;

#========================================================================================
# function     GetShopUserIDsByEMail
#----------------------------------------------------------------------------------------
# syntax       $aUserIDs = GetShopUserIDsByEMail($dbi, $EMail, $SiteID, $ClassID);
#----------------------------------------------------------------------------------------
# description  get all shop user ids by email address
#----------------------------------------------------------------------------------------
# input        $EMail |  login-email of user | email address
# input        $dbi | $dbi | object
# input        $SiteID | Site object id (e.g. shop or system id) | int
# input        $ClassID |  user class id | int
# return       $aUserIDs | list of user identifier | ref.array.integer
#========================================================================================
sub GetShopUserIDsByEMail {
    my $self = shift;
    my ($dbi, $EMail, $SiteID, $ClassID) = @_;

    my $aaRows = $dbi->execute( <<'END_SQL', [$EMail, $SiteID, $EMail, $SiteID, $ClassID] );
SELECT u.userid, o.creationdate AS CDate
FROM users u
INNER JOIN object o on o.objectid = u.userid
WHERE u.email = ?
  AND o.siteid=?
UNION
SELECT o.objectid, o.creationdate AS CDate
FROM object o
WHERE o.alias = ?
  AND o.siteid=?
  AND o.classid=?
ORDER BY CDate DESC
END_SQL
    return [ map { $_->[0] } @$aaRows ];
}

#========================================================================================
# function     GetVerifiedShopUserIDsByEMail
#----------------------------------------------------------------------------------------
# syntax       $aUserIDs = GetVerifiedShopUserIDsByEMail($dbi, $EMail, $SiteID, $ClassID);
#----------------------------------------------------------------------------------------
# description  get all shop user ids (which are veified) by email address
#----------------------------------------------------------------------------------------
# input        $EMail |  login-email of user | email address
# input        $dbi | $dbi | object
# input        $SiteID | Site object id (e.g. shop or system id) | int
# input        $ClassID |  user class id | int
# return       $aUserIDs | list of user identifier | ref.array.integer
#========================================================================================
sub GetVerifiedShopUserIDsByEMail {
    my $self = shift;
    my ($dbi, $EMail, $SiteID, $ClassID) = @_;

    my $aaRows = $dbi->execute( <<'END_SQL', [$EMail, $SiteID, $EMail, $SiteID, $ClassID] );
SELECT u.userid, o.creationdate AS CDate
FROM users u
INNER JOIN object o on o.objectid = u.userid
WHERE u.email = ?
  AND u.isemailverified = 1
  AND o.siteid=?
UNION
SELECT o.objectid, o.creationdate AS CDate
FROM object o
WHERE o.alias = ?
  AND o.siteid=?
  AND o.classid=?
ORDER BY CDate DESC
END_SQL
    return [ map { $_->[0] } @$aaRows ];
}



1;
