#========================================================================================
# package      DE_EPAGES::Presentation::DAL::Users
# state        private
# description  database interface for users:
#========================================================================================
package DE_EPAGES::Presentation::DAL::Users;
use base qw (Exporter);

use strict;
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString );
use DE_EPAGES::Database::API::Connection qw (  GetCurrentDBHandle );

our @EXPORT = qw (DAL_GetShopUserIDsByEMail DAL_GetVerifiedShopUserIDsByEMail);

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Presentation::DAL::'.$dbi->driverName.'::Users';
    DE_EPAGES::Core::API::PerlTools::UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

#========================================================================================
# function     DAL_GetShopUserIDsByEMail
#----------------------------------------------------------------------------------------
# syntax       $aShopUserIDs = DAL_GetShopUserIDsByEMail($EMail, $SiteID, $ClassID);
#----------------------------------------------------------------------------------------
# description  get shop user ids by email.
#----------------------------------------------------------------------------------------
# input        $EMail |  login-email of user | email address
# input        $SiteID | Site object id (e.g. shop or system id) | int
# input        $ClassID |  user class id | int
# return       $aShopUserIDs | shop user ids | ref.array.int
#========================================================================================
sub DAL_GetShopUserIDsByEMail ($$$) {
    my ($EMail, $SiteID, $ClassID) = @_;
    TestString('EMail', $EMail);
    TestInteger('SiteID', $SiteID);
    TestInteger('ClassID', $ClassID);


    return _CallDriver('GetShopUserIDsByEMail', $EMail, $SiteID, $ClassID);
}

#========================================================================================
# function     DAL_GetShopUserIDsByEMail
#----------------------------------------------------------------------------------------
# syntax       $aShopUserIDs = DAL_GetShopUserIDsByEMail($EMail, $SiteID, $ClassID);
#----------------------------------------------------------------------------------------
# description  get shop user ids by email.
#----------------------------------------------------------------------------------------
# input        $EMail |  login-email of user | email address
# input        $SiteID | Site object id (e.g. shop or system id) | int
# input        $ClassID |  user class id | int
# return       $aShopUserIDs | shop user ids | ref.array.int
#========================================================================================
sub DAL_GetVerifiedShopUserIDsByEMail ($$$) {
    my ($EMail, $SiteID, $ClassID) = @_;
    TestString('EMail', $EMail);
    TestInteger('SiteID', $SiteID);
    TestInteger('ClassID', $ClassID);


    return _CallDriver('GetVerifiedShopUserIDsByEMail', $EMail, $SiteID, $ClassID);
}

1;

