#========================================================================================
# package      DE_EPAGES::Product::API::Object::LineItemCustomizedFile
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for LineItemCustomizedFile.
#========================================================================================
package DE_EPAGES::Product::API::Object::LineItemCustomizedFile;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItemCustomizedFile->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'LineItemCustomizedFile' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $LineItemCustomizedFile->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemCustomizedFile = DE_EPAGES::Product::API::Object::LineItemCustomizedFile->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemCustomizedFile');
#               my $LineItemCustomizedFile = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemCustomizedFile' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemCustomizedFile | new object | object
# hook         OBJ_InsertLineItemCustomizedFile | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomizableFileID - object id            - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertLineItemCustomizedFile', {'Object' => $self, 'CustomizableFileID' => $self->id });
    delete $hInfo->{$_} foreach qw ( LineItemID FileName Width Height InnerWidth InnerHeight FromTop FromLeft ZIndex Orientation AreaID OptionID ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemCustomizedFile->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemCustomizedFile' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemCustomizedFile | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomizableFileID - object id            - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemCustomizedFile', {'Object' => $self, 'CustomizableFileID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemCustomizedFile->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemCustomizedFile' and 'OBJ_AfterUpdateLineItemCustomizedFile'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemCustomizedFile | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomizableFileID - object id            - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemCustomizedFile | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CustomizableFileID - object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItemCustomizedFile', {'Object'=>$self, 'Values'=> $hValues, 'CustomizableFileID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItemCustomizedFile',  {'Object'=>$self, 'Values'=> $hValues, 'CustomizableFileID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     clone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Clone = $LineItemCustomizedFile->clone;
#----------------------------------------------------------------------------------------
# description  Clones LineItemCustomizedFile object.
#----------------------------------------------------------------------------------------
# return       $Clone | LineItemCustomizedFile | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive, $CloneFiles) = @_;

    my $Clone;
    $Clone = $self->SUPER::clone($hInfo, $Recursive, $CloneFiles);
    $Clone->set({'LineItemID' => $Clone->parent->parent->id });
    return $Clone;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Product::API::Object::LineItemCustomizedFile->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'FileName' => '',
        'FromTop' => 0,
        'FromLeft' => 0,
        'ZIndex' => 0,
        'Orientation' => 0,
        'AreaID' => 0,
        'OptionID' => 0,
    };
}

1;
