#!/srv/epages/eproot/Perl/bin/perl
use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::PerlTools qw ( GenerateShortGUID );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Permission::DAL::Permission qw( DAL_GetUsersWithEmailInAlias );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject ExistsObject ExistsObjectByPath LoadObject JoinObjectPath);

sub Main {
    local $| = 1;
    my %Options = ();
    my @GetOpt = qw(help storename=s);
    usage() unless GetOptions(\%Options, @GetOpt);
    usage() if defined $Options{help};
    usage() unless defined $Options{storename};

    my $time1 = time;

    RunOnStore(
        Store => $Options{storename},
        Sub => sub {
            my $ShopContainer = LoadRootObject()->child('Shops');
            foreach my $ShopID (@{$ShopContainer->findChildren}) {
                next unless ExistsObject($ShopID);
                my $Shop = LoadObject($ShopID);

                next unless defined $Shop;

                my $aData = DAL_GetUsersWithEmailInAlias($ShopID);
                # iterate, get user, generate GUID, convert alias-email to alias-guid
                foreach my $hash (@{$aData}) {
                    my $User = LoadObject( $hash->{'UserId'} );

                    my $Counter = 0;
                    my $Guid;
                    my $Path;
                    do {
                        $Guid = GenerateShortGUID();

                        $Path = JoinObjectPath([
                            'Shops',
                            $Shop->alias,
                            'Users',
                            $Guid,
                        ]);

                        if($Counter > 10) {
                            Error( 'GUID AlreadyExistsForUserOnShop', { 'ObjectPath' => $Path } )
                        }

                        $Counter++;
                    }
                    while (ExistsObjectByPath($Path));

                    # set GUID into alias
                    $User->set({ 'Alias' => $Guid });
                }
            }
        }
    );

    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    This script changes alias of the shop user to short guid.

Usage:
    $0 [options] [flags]

options:
    -storename    name of store      ()

flags:
    -help         show the command line options

Example:
    \$PERL $0 -storename Store

END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;
