#========================================================================================
# package      DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorizeEvo
# base         DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for PaymentMethodSpreedlyAuthorizeEvo
#========================================================================================
package DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorizeEvo;
use base DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );

use strict;

use constant PAYMENT_TYPE_AUTHORIZE_EVO => 'SpreedlyAuthorizeEvo';

#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to view settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewSettingsSpreedlyAuthorizeEvo {
    my $self = shift;
    my ($Servlet,$PaymentMethod) = @_;

    my $Form = $Servlet->form();
    my $ApiLoginID = $Form->value('APILoginID');
    my $TransactionKey = $Form->value('TransactionKey');

    if (isValid($ApiLoginID) && isValid($TransactionKey)) {
        $self->_createGateway($ApiLoginID, $TransactionKey, $Servlet);
    }

    return $Servlet;
}

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to save settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    $Servlet->form->form($PaymentMethod, 'SpreedlyAuthorizeEvoSettings');

    $self->SUPER::SaveSettings($Servlet);

}

#---------------------------------------------------------------------------
#  Private methods
#---------------------------------------------------------------------------
sub _createGateway {
    my $self = shift;
    my ($ApiLoginID, $TransactionKey, $Servlet) = @_;

    my $hAdditionalBodyParams = {
        'api_login_id' => $ApiLoginID,
        'transaction_key' => $TransactionKey,
    };

    my $PaymentMethodSpreedly = DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly->new();
    $PaymentMethodSpreedly->SaveCredentials($Servlet, PAYMENT_TYPE_AUTHORIZE_EVO, $hAdditionalBodyParams);

    return;
}

1;
